%% tclp__errors.pl
%% Copyright (C) 2001,2002 INRIA Emmanuel Coquery

%% This library is free software; you can redistribute it and/or
%% modify it under the terms of the GNU Lesser General Public
%% License as published by the Free Software Foundation; either
%% version 2.1 of the License, or (at your option) any later version.

%% This library is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.

%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                           %%
%%                                  TCLP                                     %%
%%                                                                           %%
%%         Emmanuel Coquery, projet Contraintes, INRIA Rocquencourt          %%
%%                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Header: /home/beaune/contraintes/coquery/cvs/contraintes/tclpchr/src/tclp__errors.pl,v 1.15 2002/09/04 15:01:20 coquery Exp $

:- module(tclp__errors, [ print_error/2, 
                          print_error/1, 
                          warning/1, 
                          enable_verbose/1,
                          enable_verbose/2,
                          disable_verbose/1,
                          verbose/2,verbose_no_nl/2,
                          print_exception/1,
                          chrono/2,
                          chrono/3
                        ]).

:- use_module(library('tclp/tclp__reader')).
:- use_module(library(lists)).

%% warning(Message) emits a warning message
warning(Message) :-
        write('* Warning : '), print_list(Message), nl.

%% print_error(Location, Message)
%% prints an error message for an error that occured at location
%% if Message is a list, write its elements one after another, otherwise,
%% write it directly.
print_error(Loc, Message) :-
        line_number(Loc, Line),
        get_filename(Loc, FileName),
        print_list(['! Error in "',FileName,'", line ',Line,' : ']),
        print_list(Message), nl, throw(tclp__error).

%% print_error(Message)
%% prints an error message
%% if Message is a list, write its elements one after another, otherwise,
%% write it directly.
print_error(Message) :-
        write('! Error : '),
        print_list(Message), nl, throw(tclp__error).

%% print_time prints the time elapsed since the last time printing
print_time :-
        statistics(runtime,[_,Elapsed]),
        Display is Elapsed / 1000,
        write(Display),write(' s').

convert_var_names(X,'$VAR'(X)) :-
        atom(X),atom_codes(X,[First|_]),
        (First >= 65, First =< 90 ; char_code('_',First)),!.
convert_var_names(X,X) :- var(X),!.
convert_var_names([],[]) :- !.
convert_var_names([X|Xs],[Y|Ys]) :- !,
        convert_var_names(X,Y),
        convert_var_names(Xs,Ys).
convert_var_names(X,Y) :-
        X =.. [ H | Args ],
        convert_var_names(Args,Args2),
        Y =.. [ H | Args2 ].
        

print_simple(X) :-
        X==time, !, print_time.
print_simple(X) :-
        nonvar(X), X=quoted(Y), !,convert_var_names(Y,Z),
        write_term(Z,[quoted(true),numbervars(true)]).
print_simple(X) :- write(X).
        

%% print_list(List), prints elements of the list one after another
print_list([]):-!.
print_list([X|L]) :- !, print_simple(X), print_list(L).
print_list(Message) :- print_simple(Message).

%% verbose(Level) is true if the level Level of verbose is activated
:- dynamic verbose/1, no_comment/1.

%% verbose(Mode, Message) prints a message is the level of verbose is enabled
%% always is always enabled, printing with comment by default
verbose(Level, Message) :-
        (verbose(Level);Level==always)
        ->
        (no_comment(Level) -> true ; write('%% ')),
        print_list(Message),nl
        ; 
        true .
verbose_no_nl(Level, Message) :-
        (verbose(Level);Level==always), !,
        (no_comment(Level) -> true ; write('%% ')),
        print_list(Message),flush_output; 
        true .

enable_verbose(Level,DoComment) :-
        asserta(verbose(Level)),
        (DoComment -> true ; asserta(no_comment(Level))).

enable_verbose(Level) :-
        enable_verbose(Level,true).

disable_verbose(Level) :-
        ( var(Level), write('%% Removing all verbose levels !!!'), nl; true ),
        retractall(verbose(Level)),
        retractall(no_comment(Level)).

:- meta_predicate chrono(?,:).
chrono(Name, Goal) :-
        (
          is_list(Name)
        ->
          append(['Starting '|Name],[' ...'],Message),
          append(Name,[' done in ', Display, ' s'],Message2a),
          append(['failed '|Name],[' in ', Display, ' s'],Message2b)
        ;
          Message = ['Starting ', Name, ' ...'],
          Message2a = [Name, ' done in ', Display, ' s'],
          Message2b = ['failed ',Name, ' in ', Display, ' s']
        ),
        verbose(timing,Message),
        statistics(runtime,[FromStart,_]),
        catch((call(Goal),Result=true;Result=fail),E,Result=throw(E)),
        statistics(runtime,[FromStart2,_]),
        Elapsed is FromStart2 - FromStart,
        Display is Elapsed / 1000,
        ((Result=fail ; Result=throw(_)) ->
            Message2 = Message2b ; Message2a = Message2),
        verbose(timing,Message2),
        call(Result).

:- meta_predicate chrono(?,?,:).
chrono(VerboseLevel, Name, Goal) :-
        verbose(VerboseLevel) -> chrono(Name,Goal) ; Goal.

print_syntax_tokens([]).
print_syntax_tokens([ atom(X)-_ | Rest ]) :- !,
        write_term(X,[quoted(true)]),write(' '),
        print_syntax_tokens(Rest).
print_syntax_tokens([ var(_,Codes,_)-_ | Rest]) :- !,
        atom_codes(Name,Codes),
        write(Name), write(' '),
        print_syntax_tokens(Rest).
print_syntax_tokens([X-_ | Rest ]) :-
        write(X),write(' '),
        print_syntax_tokens(Rest).

%% pretty prints errors
print_exception(tclp__error_in_file(FileName,
                                    syntax_error(_,Line,Description,
                                                 Terms,NbRemaining))) :-
        length(Trailing, NbRemaining),!,
        append(Start,Trailing,Terms),
        write('! Syntax error'),nl,
        write('! '),write(Description),nl,
        write('! '),write('in file '),write_term(FileName,[quoted(true)]),
        write(', line '), write(Line),nl,
        write('! '),print_syntax_tokens(Start),nl,
        write('! *HERE*'),nl,
        write('! '),print_syntax_tokens(Trailing),nl.
print_exception(E) :-
        write_term(E,[quoted(true)]),nl.
