%% tclp__reader.pl
%% Copyright (C) 2001,2002 INRIA Emmanuel Coquery

%% This library is free software; you can redistribute it and/or
%% modify it under the terms of the GNU Lesser General Public
%% License as published by the Free Software Foundation; either
%% version 2.1 of the License, or (at your option) any later version.

%% This library is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%% Lesser General Public License for more details.

%% You should have received a copy of the GNU Lesser General Public
%% License along with this library; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                                           %%
%%                                  TCLP                                     %%
%%                                                                           %%
%%         Emmanuel Coquery, projet Contraintes, INRIA Rocquencourt          %%
%%                                                                           %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% $Header: /home/beaune/contraintes/coquery/cvs/contraintes/tclpchr/src/tclp__reader.pl,v 1.14 2002/06/26 08:30:43 coquery Exp $

%:- use_module(tclp__utils).
%:- use_module(tclp__def).

:- module(tclp__reader, [read_one_term/4,
                         args_data/3,
                         variable_name/3,
                         line_number/2,
                         get_filename/2,
                         get_location/2,
                         term_data_from_variable_names/2]).

%% This module provides utilities to read terms
%% The type of data attached to read terms

%% :- type term_data is [ term_data(term, list(pair(term,atom)), list(term), atom) ].


%% :- pred convert_to_pairs(list(bool),list(pair(term,atom))).
%% converts a list of elements X=Y to a list of pairs X-Y
convert_to_pairs([],[]).
convert_to_pairs([(X=Y)|L],[X-Y|R]) :- convert_to_pairs(L,R).

%% :- pred convert_to_var_list(list(bool),list(term)).
%% converts a list of elements X=Y to a list of X
convert_to_var_list([],[]).
convert_to_var_list([(X=_)|L],[X|R]) :- convert_to_var_list(L,R).

%% :- pred read_one_term(stream, term, term_data).
%% parse one term from the stream and returns the corresponding term_data
read_one_term(Stream, Term, TermData, FileName) :-
        catch(read_term(Stream, Term,
                        [ layout(Layout),
                          variable_names(VariableNames),
                          singletons(Singletons),
                          syntax_errors(error)]),
              E,
              throw(tclp__error_in_file(FileName,E))
             ),
        convert_to_pairs(VariableNames, VariableNames2),
        convert_to_var_list(Singletons, Singletons2),
        TermData = term_data(Layout, VariableNames2, Singletons2, FileName).

%% eq_list(L) unifies all elements of L
eq_list([]).
eq_list([_]).
eq_list([X,X|L]) :- eq_list([X|L]).

%% Wrapper for args_data
user:args_location(TermData,ArgsData) :-
        args_data(TermData,ArgsData,_).

%% :- pred args_data(term_data, list(term_data), int).
%% term_data corresponding to the sub terms of a given compound term
args_data(term_data([_|ArgsLayout], VariableNames, Singletons, FileName),
          ArgsData, Length) :-
        length(ArgsLayout, Length),!,
        args_data_list(ArgsLayout,
                       VariableNames, Singletons, FileName,
                       ArgsData).
args_data(term_data([Line|_], VariableNames, Singletons, FileName),
          ArgsData, Length) :- !,
        length(ArgsLayout, Length),
        eq_list([Line|ArgsLayout]),
        args_data_list(ArgsLayout,
                       VariableNames, Singletons, FileName,
                       ArgsData).
args_data(term_data(Line, VariableNames, Singletons, FileName),
          ArgsData, Length) :- !,
        length(ArgsLayout, Length),
        eq_list([Line|ArgsLayout]),
        args_data_list(ArgsLayout,
                       VariableNames, Singletons, FileName,
                       ArgsData).
args_data(term_data(_,_,_,_),[],0).

%% :- pred args_data_list(list(term), list(pair(term,term)),
%%                        list(pair(term,term)), list(term_data)).
%% transforms a list of layouts into a list of term_data using the given
%% VariableNames and Singletons
args_data_list([],_,_,_,[]).
args_data_list([Layout | Layouts], VariableNames, Singletons, FileName,
               [ term_data(Layout, VariableNames,Singletons, FileName)
               | TermDatas ]) :-
        args_data_list(Layouts, VariableNames, Singletons, FileName, TermDatas).

%% This has been replaced by tclp__handle_goal
%%% :- pred exec_term(term).
%%% execute the given term if its a recognized primitive
%exec_term((:- op(X,Y,Z))) :- !,op(X,Y,Z).
%exec_term(_).

%% :- pred variable_name(term, term_data, atom)
%% unify the given variable with the atom corresponding to its name.
variable_name(V,term_data(_,VariableNames,_,_), Name) :-
        variable_name2(V, VariableNames, Name).
variable_name2(V, [N-X|_], Name) :-
        V == X , Name = N,!.
variable_name2(V, [_|Names], Name) :-
        variable_name2(V,Names,Name).


%% :- pred line_number(term_data, int)
%% unify LineNumber with the number of the line in the layout of the term_data
line_number(term_data([LineNumber|_],_,_,_), LineNumber).
line_number(term_data(LineNumber,_,_,_), LineNumber).

%% :- pred get_filename(term_data, atom)
%% unify FileName with the name of the file corresponding to this term data
get_filename(term_data(_,_,_,FileName), FileName).

%% the first argument is a term_data and the second argument is an atom
%% describing the file and the line corresponding to this term_data
get_location(TermData,Location) :-
        line_number(TermData,Line),
        get_filename(TermData,FileName),
        tclp__utils:convert_list_to_atom([FileName,' : line ',Line],Location).

%% term_data_from_variable_names builds a term_data term with nothing except
%% the variablenames which is given by the first argument
term_data_from_variable_names(Names,term_data(_,Names,_,_)).
